from jobx.task import Task
import textwrap

from .registry import register,TaskBuilder
@register(["python-code","py-code"])
class PythonCodeRunner(TaskBuilder):
    def match(self,task:dict):
        return "python-code" in task or "py-code" in task
    def build(self,task:dict, name, g=None):
        code = task.get("py-code",task.get("python-code"))
        env = task.get('env',{})
        code=textwrap.dedent(code)
        def task():
            exec(code)
        return Task(task, name, g)




import os
from jobx.utils import  generate_env_string
from jobx.task import Task

from .registry import register,TaskBuilder
@register(["command","cmd"])
class Command(TaskBuilder):
    def match(self,task:dict):
        return "cmd" in task or "command" in task
    def build(self,task:dict, name, g=None):
        command = task.get("cmd",task.get("command"))
        env = task.get('env',{})
        def task():
            os.system(generate_env_string(**env)+command)
        return Task(task, name, g)




from jobx.utils import import_module, generate_env_string
from jobx.task import Task
import os
import sys
from .registry import register,TaskBuilder
@register(["python","py"])
class PythonScript(TaskBuilder):
    type="python"
    def match(self,task:dict):
        return "py-path" in task
    def build(self,task:dict, name, g=None):
        python_file = task.get("py-path",task.get("path"))
        args = task.get('args',[])
        kwargs = task.get('kwargs',{})
        func_name= task.get('function',task.get("func",None))
        env=task.get('env',{})
        if func_name:
            module = import_module(python_file)
            func = getattr(module, func_name)
            task = lambda: func(*args, **kwargs)
            return Task(task, name, g)
        else:
            args = ' '.join([str(x) for x in args])
            kwargs_str = []
            for k, v in kwargs.items():
                if isinstance(v, bool):
                    if v:
                        kwargs_str.append('--%s' % (k))
                    else:
                        continue
                else:
                    kwargs_str.append('--%s=%s' % (k, v))
            kwargs = ' '.join(kwargs_str)

            def task():
                os.system(generate_env_string(**env)+f"{sys.executable} {python_file}{' ' + args if args else ''}{' ' + kwargs if kwargs else ''}")
            return Task(task, name)



import  time

class Task:
    def __init__(self, target, name, g=None) -> None:
        if g is None:
            g={}
        self.target = target
        self.started = False
        self.finished = False
        self.name = name
        self.g=g
        self.on_started = None
        self.on_finished = None

    def set_on_started(self, func):
        self.on_started = func

    def set_on_finished(self, func):
        self.on_finished = func

    def start(self):
        g=self.g
        self.started = True
        if self.on_started:
            self.on_started(self)
        if g.get("timer", False):
            start_time = time.time()
            print(("Task %s started." % (self.name)).center(100, "="))
        self.target()
        if g.get("timer", False):
            end_time = time.time()
            print(("Task %s finished. Time used: %.5f" % (self.name, end_time - start_time)).center(100, "="))
        if self.on_finished:
            self.on_finished(self)
        self.finished = True




from fnmatch import fnmatch
from .registry import register,TaskBuilder,match_task
from ..task import Task

def build_task(task, name, g=None):
    assert isinstance(task, dict)
    builder=match_task(task)
    tasks = builder.build(task, name,g)
    if isinstance(tasks, Task):
        tasks = [tasks]
    return tasks

@register("task-group")
class TaskGroup(TaskBuilder):
    def match(self,task:dict):
        return "tasks" in task
    def build(self,cfg:dict, name, g=None, task_spec=None):
        '''
        task_spec example: 'train-*,test-*'
        '''
        if g is None:
            g = {}
        task_dict = cfg.get('tasks')
        # print(task_dict.keys())
        if isinstance(task_dict, (list, tuple, set)):
            task_dict = {str(k): v for k, v in enumerate(task_dict)}
        def search_keys(pattern, candidates):
            if not {',', '*'}.intersection(set(list(pattern))):
                if pattern in candidates:
                    # print(pattern)
                    return [pattern]
                else:
                    return None
            results = []
            if ',' in pattern:
                for p in pattern.split(','):
                    p = p.strip()
                    r = search_keys(p, candidates)
                    if r is not None:
                        results += r
            for cand in candidates:
                if fnmatch(cand, pattern) and not cand in results:
                    results.append(cand)
            return results

        if task_spec:
            keys = search_keys(str(task_spec), list(task_dict.keys()))
            if not keys:
                print(f"No task matches : {task_spec}")
                return []
            task_dict = {key: task_dict[key] for key in keys}

        task_dict = {f'{name}-{key}': v for key, v in task_dict.items()}
        task_list = []
        for name, t in task_dict.items():
            # print(name)
            task_list += build_task(t, name, g)
        return task_list


